@@ -9,7 +9,8 @@
 #
 ac_help='--enable-amalloc	Enable memory allocation debugging
 --with-tabstops=N	Set tabstops to N characters (default is 4)
---shared		Build shared libraries (default is static)'
+--shared		Build shared libraries (default is static)
+--pkg-config		Install pkg-config(1) glue files'
 
 LOCAL_AC_OPTIONS='
 set=`locals $*`;
@@ -31,6 +32,9 @@ locals() {
     --DEBIAN-GLITCH)
 		echo DEBIAN_GLITCH=T
 		;;
+    --PKG-CONFIG)
+		echo PKGCONFIG=true
+		;;
     esac
 }
 
@@ -63,6 +67,7 @@ AC_DEFINE THEME_CF "$THEME_CF"
 test "$DEBIAN_GLITCH" && AC_DEFINE 'DEBIAN_GLITCH' 1
 
 AC_PROG_CC
+AC_CHECK_NORETURN
 
 test "$TRY_SHARED" && AC_COMPILER_PIC && AC_CC_SHLIBS
 
@@ -85,6 +90,17 @@ fi
 AC_PROG ar || AC_FAIL "$TARGET requires ar"
 AC_PROG ranlib
 
+# should we create a .pc for pkg-config & GNU automake
+#
+if [ "$PKGCONFIG" ]; then
+    AC_SUB MK_PKGCONFIG ''
+elif AC_PROG pkg-config || AC_PROG automake ; then
+    PKGCONFIG=true
+    AC_SUB MK_PKGCONFIG ''
+else
+    AC_SUB MK_PKGCONFIG '#'
+fi
+
 AC_C_VOLATILE
 AC_C_CONST
 AC_C_INLINE
@@ -162,4 +178,10 @@ fi
 
 [ "$WITH_PANDOC_HEADER" ] && AC_DEFINE 'PANDOC_HEADER' '1'
 
-AC_OUTPUT Makefile version.c mkdio.h libmarkdown.pc
+GENERATE="Makefile version.c mkdio.h"
+
+if [ "$PKGCONFIG" ]; then
+    GENERATE="$GENERATE libmarkdown.pc"
+fi
+
+AC_OUTPUT $GENERATE
